import base64
import io

import cv2
import numpy as np
import numpy.typing as npt

from PIL import Image, ImageOps


def array2bytes(array: npt) -> bytes:
    """Convert a numpy array to bytes"""
    return array.tobytes()


def bytes2array(byte: bytes, dtype: str = "uint64") -> npt:
    """Convert a byte stream into a numpy array"""
    return np.frombuffer(byte, dtype=dtype)


def pil2cv2(pil: Image):
    pil_image = pil.convert("RGB")
    cv_image = np.array(pil_image, dtype=np.uint8)
    # Convert RGB to BGR
    cv_image = cv2.cvtColor(cv_image, cv2.COLOR_RGB2BGR)
    return cv_image


def cv22pil(image, color_format: str = ""):
    # Convert the color format (if needed)
    if color_format.lower() == "bgr":
        image = cv2.cvtColor(image, cv2.COLOR_BGR2RGB)
    elif color_format.lower() == "hsv":
        image = cv2.cvtColor(image, cv2.COLOR_HSV2RGB)
    elif color_format.lower() == "lab":
        image = cv2.cvtColor(image, cv2.COLOR_LAB2RGB)
    # Create a PIL image from the numpy array
    im_pil = Image.fromarray(image)
    return im_pil


def pil2bytes(pil: Image) -> bytes:
    """Convert a numpy array to bytes"""
    img_bytes = io.BytesIO()
    pil.save(img_bytes, format="JPEG", quality=100, subsampling=0)
    return img_bytes.getvalue()


def bytes2pil(bytes_data, using_index=22, is_flip=True):
    if using_index > 0:
        bytes_data = bytes_data[using_index:]

    image = Image.open(io.BytesIO(base64.b64decode(bytes_data)))
    if is_flip:
        image = ImageOps.mirror(image)
    image_new = Image.new("RGB", image.size, (255, 255, 255))
    image_new.paste(image, mask=image.split()[3])

    return image_new


def cv22bytes(image: npt.NDArray[np.uint8]) -> bytes:
    return cv2.imencode(
        ".jpg",
        image,
        [cv2.IMWRITE_JPEG_QUALITY, 100, cv2.IMWRITE_JPEG_SAMPLING_FACTOR, cv2.IMWRITE_JPEG_SAMPLING_FACTOR_444],
    )[1].tobytes()


def bytes2cv2(bytes_data, using_index=22, is_flip=True):
    pil_image = bytes2pil(bytes_data, using_index, is_flip)
    return pil2cv2(pil_image)



import csv
import gzip
import hashlib
import json
import os
import pickle
import re
import tarfile
import warnings

from typing import IO, Any, Dict, Generator


def text_reader(path: str) -> str:
    with open(path, "r") as fp:
        return fp.read()


def text_line_reader(path: str) -> Generator[str, Any, None]:
    with open(path, "r") as fp:
        for line in fp:
            line = line.strip()
            if line:
                yield line


def text_writer(path: str, data: str) -> None:
    with open(path, "w") as fwrite:
        fwrite.write(data)


def raw_reader(path: str) -> bytes:
    with open(path, "rb") as fp:
        bin_data = fp.read()
    return bin_data


def csv_line_reader(path: str, delimiter: str, skip_header: bool) -> Generator[Any, Any, None]:
    with open(path, "r") as fp:
        reader = csv.reader(fp, delimiter=delimiter)

        if skip_header:
            next(reader)

        for row in reader:
            yield row


def gzip_text_line_reader(path: str) -> Generator[str, Any, None]:
    with gzip.open(path, "rt") as fp:
        for line in fp:
            line = line.strip()
            if line:
                yield line


def tar_gzip_file_reader(path: str, pattern: str) -> Generator[IO, Any, None]:
    pattern = re.compile(pattern)
    tarf = tarfile.open(path, "r:gz")
    for filename in tarf.getnames():
        if re.search(pattern, filename):
            fp = tarf.extractfile(filename)
            yield fp


def json_reader(path: str) -> Dict[str, Any]:
    with open(path, "r") as fp:
        data = json.load(fp)
    return data


def json_writer(path: str, data: Any, indent=None) -> None:
    with open(path, "w") as fp:
        json.dump(data, fp, indent=indent)


def get_md5_file(path):
    bin_data = raw_reader(path)
    return hashlib.md5(bin_data).hexdigest()


def dump_pickle(obj):
    """
    Serialize an object.

    Returns :
        The pickled representation of the object obj as a bytes object
    """
    return pickle.dumps(obj)


def delete_file(file_path):
    if os.path.isfile(file_path):
        os.remove(file_path)
    else:
        warnings.warn(f"{file_path} Not a file")



import os

from glob import glob

from lmdbsystem.lmdb import Lmdb
from lmdbsystem.read_adapters.text import TextReadAdapter
from tqdm import tqdm

from .path import get_relative_path


def compare_txt_lmdb_file(src_file, dest_file):
    src_adapter = TextReadAdapter(path=src_file)
    dest_adapter = TextReadAdapter(path=dest_file)

    if src_adapter.length != dest_adapter.length:
        raise ValueError(
            f"Number key of source {os.path.basename(src_file)}: {src_adapter.length} "
            f"different "
            f"Number key of destination {os.path.basename(dest_file)}: {dest_adapter.length}"
        )

    src_lmdb = Lmdb(adapter=src_adapter)
    dest_lmdb = Lmdb(adapter=dest_adapter)
    for i, key in enumerate(tqdm(src_adapter.keys)):
        src_value = src_lmdb.read_key(key)
        dest_value = dest_lmdb.read_key(dest_adapter.keys[i])
        if src_value != dest_value:
            raise ValueError(
                f"Content of source {src_file}: {src_value} "
                f"different"
                f"Content of destination {dest_file}: {dest_value}"
            )


def compare_file(src_file, dest_file):
    content_src = open(src_file, "r").read()
    content_dest = open(dest_file, "r").read()
    if content_src != content_dest:
        print(
            f"Content of source {src_file}: {content_src[:50]} "
            f"different"
            f"Content of destination {dest_file}: {content_dest[:50]}"
        )


def compare_folder(src_dir, dest_dir, suffix):
    path_dest_misses = []
    content_dest_misses = []
    content_src_misses = []
    list_src = sorted(glob(f"{src_dir}/**/*{suffix}", recursive=True))

    for i, path_src in tqdm(enumerate(list_src)):
        relative_path = get_relative_path(src_dir, path_src)
        path_dest = os.path.join(dest_dir, relative_path)

        if not os.path.isfile(path_dest):
            if open(path_src, "r").read() != "{}":
                path_dest_misses.append(path_dest)
                print(
                    f"Not found destination file {path_dest}\n"
                    f"source file has value: {open(path_src, 'r').read()[:50]}"
                )
            continue

        content_src = open(path_src, "r").read()
        content_dest = open(path_dest, "r").read()
        if content_src != content_dest:
            if content_dest.strip() == "{}":
                content_dest_misses.append(path_dest)

            if content_src.strip() == "{}":
                content_src_misses.append(path_src)
            print(
                f"Content of source {path_src}: {content_src[:50]} "
                f"different\n"
                f"Content of destination {path_dest}: {content_dest[:50]}"
            )

    print("PATH DESTINATION MISSES", path_dest_misses)
    print("############################")
    print("PATH SOURCE MISSES", content_src_misses)
    print("############################")
    print("CONTENT DESTINATION MISSES", content_dest_misses)



def str2bytes(string: str, encoding: str = "utf-8") -> bytes:
    """Convert a string to bytes"""
    return string.encode(encoding)


def bytes2str(byte: bytes) -> str:
    """Convert a byte stream into a string"""
    return byte.decode("utf-8")



import pytest

from data_helper import path


@pytest.mark.parametrize(
    "directory,file_path,expected",
    (
        ("/tmp/abcd", "/tmp/abcd/def.jpg", "def.jpg"),
        ("/tmp/abcdr", "/tmp/abcd/def.jpg", "../abcd/def.jpg"),
    ),
)
def test_get_relative_path(directory: str, file_path: str, expected: str):
    assert path.get_relative_path(directory, file_path) == expected


@pytest.mark.parametrize(
    "file_path,expected",
    (
        ("/tmp/abcd/def.jpg/", "/tmp/abcd/def.jpg"),
        ("/tmp//abcd//def.jpg", "/tmp/abcd/def.jpg"),
    ),
)
def test_normalize_path(file_path: str, expected: str):
    assert path.normalize_path(file_path) == expected


@pytest.mark.parametrize(
    "file_path,expected",
    (("/tmp/abcd/def.jpg", "/tmp/abcd/def"),),
)
def test_removesuffix_path(file_path: str, expected: str):
    assert path.removesuffix_path(file_path) == expected


@pytest.mark.parametrize(
    "file_path,expected",
    (("/tmp/abcd/def.jpg", "/tmp/abcd"),),
)
def test_get_dirname(file_path: str, expected: str):
    assert path.get_dirname(file_path) == expected


@pytest.mark.parametrize(
    "file_path,expected",
    (("/tmp/abcd/def.jpg", "def.jpg"),),
)
def test_get_filename(file_path: str, expected: str):
    assert path.get_filename(file_path) == expected



import os


def get_relative_path(directory: str, file_path: str):
    file_name = os.path.relpath(file_path, directory)
    return file_name


def normalize_path(file_path: str):
    return os.path.normpath(file_path)


def removesuffix_path(file_path: str):
    return file_path.rsplit(".", 1)[0]


def get_dirname(file_path: str):
    return os.path.dirname(file_path)


def get_filename(file_path: str):
    return os.path.basename(file_path)
